package Glib::Object::Introspection;

# $Id: Introspection.pm,v 1.1.1.1 2005/08/21 14:39:45 torsten Exp $

use 5.008;
use strict;
use warnings;

use Carp qw(croak);
use Glib;

require DynaLoader;

our @ISA = qw(DynaLoader);
our $VERSION = '0.01';

sub dl_load_flags { $^O eq 'darwin' ? 0x00 : 0x01 }

Glib::Object::Introspection -> bootstrap($VERSION);

# --------------------------------------------------------------------------- #

our $AUTOLOAD;
our $REPOSITORY;
our $BASENAME;
our $PACKAGE;

sub setup {
  my ($self, $library, $raw_idl, $basename, $package) = @_;

  $BASENAME = $basename;
  $PACKAGE = $package;

  # Load the library
  my $so = DynaLoader::dl_findfile($library);
  DynaLoader::dl_load_file($so, 0x01);

  # Load the metadata
  eval {
    $REPOSITORY = Glib::Object::Introspection::Repository -> get_default();

    open RAW, "<", $raw_idl or croak "Couldn't open raw idl file: $!";
      my $metadata;
      while (<RAW>) {
        $metadata .= $_;
      }
    close RAW;

    $REPOSITORY -> register($metadata);
    $REPOSITORY -> register_types($basename => $package);
  };

  no strict "refs";

  # Create the class structure
  foreach my $info ($REPOSITORY -> get_infos($basename)) {
    if ($info -> isa("Glib::Object::Introspection::ObjectInfo")) {
      my $name = $info -> get_name();
      @{$package . "::" . $name . "::ISA"} = ($package, "Glib::Object");
    }

    elsif ($info -> isa("Glib::Object::Introspection::BoxedInfo")) {
      my $name = $info -> get_name();
      @{$package . "::" . $name . "::ISA"} = ($package, "Glib::Boxed");
    }
  }

  if ($@) {
    croak "An error occured during IDL loading: $@";
  }
}

sub AUTOLOAD {
  (my $symbol = $AUTOLOAD) =~ s/${PACKAGE}:://;

  if ($symbol =~ m/(.+)::(.+)/) {
    my ($namespace, $method) = ($1, $2);

    my $object = $REPOSITORY -> find_by_name($BASENAME, $namespace);
    my $type = $object -> get_type();

    if ($type eq "object" || $type eq "boxed") {
      my $info;

      if ($info = $object -> find_method($method)) {
        return $info -> get_flags() & "is-constructor" ?
          $info -> invoke(@_[1..$#_]) :
          $info -> invoke(@_);
      }

      elsif ($info = grep { $_ -> get_name() eq $method }
                          $object -> get_fields()) {
        croak "FIXME: Implement fields handling";
      }

      else {
        croak "Dispatch error: $PACKAGE::$symbol doesn't exist";
      }
    } else {
      croak "Unhandled type: $type";
    }
  }

  croak "Invalid invocation: Cannot handle $AUTOLOAD";
}

1;

# --------------------------------------------------------------------------- #

__END__

=head1 NAME

Glib::Object::Introspection - Perl bindings for GLib's introspection library

=head1 SYNOPSIS

  # ...

=head1 ABSTRACT

Automatically generate Perl bindings from vendor-supplied IDL files!

=head1 AUTHORS

=over

=item Torsten Schoenfeld, E<lt>kaffeetisch at gmx dot deE<gt>

=item muppet, E<lt>scott at asofyet dot orgE<gt>

=back

=head1 COPYRIGHT

Copyright (C) 2005 by the gtk2-perl team

=cut
