#!/usr/bin/perl
use strict;
use warnings;

use ExtUtils::MakeMaker;
use Cwd qw(cwd);

# $Id: Makefile.PL,v 1.1.1.1 2005/08/21 14:39:45 torsten Exp $

our %build_reqs = (
  'perl-ExtUtils-Depends'   => '0.205',
  'perl-ExtUtils-PkgConfig' => '1.07',
  'perl-Glib'               => '1.090',
);

our %pre_reqs = (
  'ExtUtils::Depends'   => $build_reqs{'perl-ExtUtils-Depends'},
  'ExtUtils::PkgConfig' => $build_reqs{'perl-ExtUtils-PkgConfig'},
  'Glib'                => $build_reqs{'perl-Glib'},
);

unless (eval "use ExtUtils::Depends '$build_reqs{'perl-ExtUtils-Depends'}';"
           . "use ExtUtils::PkgConfig '$build_reqs{'perl-ExtUtils-PkgConfig'}';"
           . "use Glib '$build_reqs{'perl-Glib'}';"
           . "use Glib::MakeHelper;"
           . "use Glib::CodeGen;"
           . "1") {
  warn "$@\n";
  WriteMakefile(
    PREREQ_FATAL => 1,
    PREREQ_PM    => \%pre_reqs,
  );
  exit 1; # not reached
}

# --------------------------------------------------------------------------- #

mkdir 'build', 0777;

my %pkgcfg = ExtUtils::PkgConfig->find("gobject-introspection");

my $includedir = ExtUtils::PkgConfig->variable("gobject-introspection",
                                               "includedir");
my $header = $includedir . "/glib-2.0/gobject-introspection/girepository.h";

system(q(glib-mkenums --fhead "#ifndef __GIDLPERL_GTYPES_H__\n" ) .
                    q(--fhead "#define __GIDLPERL_GTYPES_H__ 1\n\n" ) .
                    q(--fhead "#include <glib-object.h>\n\n" ) .
                    q(--fhead "G_BEGIN_DECLS\n\n" ) .
                    q(--eprod "#define GIDLPERL_TYPE_@ENUMSHORT@ gidlperl_@enum_name@_get_type()\n" ) .
                    q(--eprod "GType gidlperl_@enum_name@_get_type (void);\n" ) .
                    q(--ftail "G_END_DECLS\n\n" ) .
                    q(--ftail "#endif /* __GIDLPERL_GTYPES_H__ */\n" ) .
                    "$header > build/gobject-introspection-perl-types.h");

system(q(glib-mkenums --fhead "#include \"girepository.h\"\n" ) .
                    q(--vhead "static const G@Type@Value _gidlperl_@enum_name@_values[] = {" ) .
                    q(--vprod "  { @VALUENAME@, \"@VALUENAME@\", \"@valuenick@\" }," ) .
                    q(--vtail "  { 0, NULL, NULL }\n};\n\n" ) .
                    q(--vtail "GType\ngidlperl_@enum_name@_get_type (void)\n{\n" ) .
                    q(--vtail "  static GType type = 0;\n\n" ) .
                    q(--vtail "  if (!type)\n" ) .
                    q(--vtail "    type = g_@type@_register_static (\"@EnumName@\", _gidlperl_@enum_name@_values);\n\n" ) .
                    q(--vtail "  return type;\n}\n\n" ) .
                    "$header > build/gobject-introspection-perl-types.c");

# --------------------------------------------------------------------------- #

our @xs_files = <xs/*.xs>;
our %pod_files = (
  'Introspection.pm' => '$(INST_MAN3DIR)/Glib::Object::Introspection.$(MAN3EXT)',
  Glib::MakeHelper->do_pod_files (@xs_files),
);

Glib::CodeGen->parse_maps ('gobject-introspection-perl');
Glib::CodeGen->write_boot (ignore => qr/^Glib::Object::Introspection$/);

my $idl = ExtUtils::Depends->new('Glib::Object::Introspection', 'Glib');
$idl->set_inc($pkgcfg{cflags} . " -Ibuild");
$idl->set_libs($pkgcfg{libs});
$idl->add_c(qw(build/gobject-introspection-perl-types.c));
$idl->add_xs(@xs_files);
$idl->add_pm('Introspection.pm' => '$(INST_LIBDIR)/Introspection.pm');
$idl->add_typemaps(map { cwd() . "/$_" }
                     qw(gobject-introspection.typemap
                        build/gobject-introspection-perl.typemap));
$idl->install(qw(gobject-introspection-perl.h
                 gobject-introspection.typemap
                 build/gobject-introspection-perl.typemap
                 build/gobject-introspection-perl-types.h));
$idl->save_config('build/IFiles.pm');

# --------------------------------------------------------------------------- #

WriteMakefile(
  NAME          => 'Glib::Object::Introspection',
  VERSION_FROM  => 'Introspection.pm',
  ABSTRACT_FROM => 'Introspection.pm',
  PREREQ_PM     => \%pre_reqs,
  XSPROTOARG    => '-noprototypes',
  MAN3PODS      => \%pod_files,
  $idl->get_makefile_vars,
);

sub MY::postamble {
  return Glib::MakeHelper->postamble_clean ()
       . Glib::MakeHelper->postamble_docs_full (
           DEPENDS => $idl,
           COPYRIGHT_FROM => 'copyright.pod');
}
