/*
 * Copyright (C) 2005 by the gtk2-perl team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: IdlRepository.xs,v 1.1.1.1 2005/08/21 14:39:45 torsten Exp $
 */

#include "gobject-introspection-perl.h"
#include <dlfcn.h>
#include "ffi.h"

GType
gidlperl_get_gtype (GIRegisteredTypeInfo *info)
{
	const gchar *symbol;
	gpointer func;
	ffi_type *return_type;
	ffi_cif cif;
	GType return_value = 0;

	symbol = g_registered_type_info_get_type_init (
			(GIRegisteredTypeInfo *) info);
	func = dlsym (NULL, symbol);
	if (!func)
		croak ("Couldn't retrieve function for %s: %s",
		       symbol, dlerror ());

	/* GType = gulong */
	return_type = &ffi_type_ulong;

	if (FFI_OK != ffi_prep_cif (&cif, FFI_DEFAULT_ABI, 0, return_type, NULL))
		croak ("Couldn't prepare function call");

	ffi_call (&cif, func, &return_value, NULL);
	if (!return_value)
		croak ("Couldn't retrieve type");

	return return_value;
}

MODULE = Glib::Object::Introspection::Repository	PACKAGE = Glib::Object::Introspection::Repository	PREFIX = g_irepository_

# GIRepository * g_irepository_get_default (void);
GIRepository_noinc *
g_irepository_get_default (class)
    C_ARGS:
	/* void */

void g_irepository_register (GIRepository *repository, const guchar *metadata);

void g_irepository_unregister (GIRepository *repository, const guchar *metadata);

GIBaseInfo * g_irepository_find_by_name (GIRepository *repository, const gchar  *namespace, const gchar  *name);

# gchar ** g_irepository_get_namespaces (GIRepository *repository);
void
g_irepository_get_namespaces (repository)
	GIRepository *repository
    PREINIT:
	gchar **namespaces = NULL;
	gchar *namespace = NULL;
    PPCODE:
	namespaces = g_irepository_get_namespaces (repository);
	while ((namespace = *namespaces++)) {
		XPUSHs (sv_2mortal (newSVGChar (namespace)));
		g_free (namespace);
	}

# FIXME: Needed?
# GIBaseInfo * g_irepository_find_by_gtype (GIRepository *repository, GType gtype);

# gint g_irepository_get_n_infos (GIRepository *repository, const gchar *namespace);
# GIBaseInfo * g_irepository_get_info (GIRepository *repository, const gchar *namespace, gint index);
void
g_irepository_get_infos (repository, namespace)
	GIRepository *repository
	const gchar *namespace
    PREINIT:
	gint number, i;
    PPCODE:
	number = g_irepository_get_n_infos (repository, namespace);
	EXTEND (sp, number);
	for (i = 0; i < number; i++)
		PUSHs (sv_2mortal (newSVGIBaseInfo (
		  g_irepository_get_info (repository, namespace, i))));

void
register_types (repository, namespace, package)
	GIRepository *repository
	const gchar *namespace
	const gchar *package
    PREINIT:
	gint number, i;
    CODE:
	number = g_irepository_get_n_infos (repository, namespace);
	for (i = 0; i < number; i++) {
		GIBaseInfo *info = g_irepository_get_info (repository, namespace, i);
		GIInfoType type = g_base_info_get_type (info);

		/* FIXME: Refactor. */
		switch (type) {
		    case GI_INFO_TYPE_OBJECT:
		    {
			const gchar *name;
			gchar *full_package;
			GType type;

			type = gidlperl_get_gtype ((GIRegisteredTypeInfo *) info);

			name = g_base_info_get_name (info);
			full_package = g_strconcat (package, "::", name, NULL);

			gperl_register_object (type, full_package);

			g_free (full_package);
		    }
			break;

		    case GI_INFO_TYPE_BOXED:
		    {
			const gchar *name;
			gchar *full_package;
			GType type;

			type = gidlperl_get_gtype ((GIRegisteredTypeInfo *) info);

			name = g_base_info_get_name (info);
			full_package = g_strconcat (package, "::", name, NULL);

			gperl_register_boxed (type, full_package, NULL);

			g_free (full_package);
		    }
			break;

		    case GI_INFO_TYPE_ENUM:
		    case GI_INFO_TYPE_FLAGS:
		    {
			const gchar *name;
			gchar *full_package;
			GType type;

			type = gidlperl_get_gtype ((GIRegisteredTypeInfo *) info);

			name = g_base_info_get_name (info);
			full_package = g_strconcat (package, "::", name, NULL);

			gperl_register_fundamental (type, full_package);

			g_free (full_package);
		    }
			break;

		    default:
			break;
		}
	}
